@@ -77,12 +77,12 @@ pid_t getsid(pid_t pid);
 #define max(x,y) (((x) > (y)) ? (x) : (y))
 #define min(x,y) (((x) < (y)) ? (x) : (y))
 
-int jvmOut = -1;
 
 /* Define a global pipe descriptor so that we don't have to keep allocating
  *  a new pipe each time a JVM is launched. */
-int pipedes[2];
-int pipeInitialized = 0;
+int pipedes[2] = {-1, -1};
+#define PIPE_READ_END 0
+#define PIPE_WRITE_END 1
 
 TCHAR wrapperClasspathSeparator = TEXT(':');
 
@@ -224,7 +224,8 @@ void takeSignalAction(int sigNum, const TCHAR *sigName, int mode) {
                 (wrapperData->jState == WRAPPER_JSTATE_STOPPED) ||
                 (wrapperData->jState == WRAPPER_JSTATE_KILLING) ||
                 (wrapperData->jState == WRAPPER_JSTATE_KILL) ||
-                (wrapperData->jState == WRAPPER_JSTATE_DOWN_CHECK)) {
+                (wrapperData->jState == WRAPPER_JSTATE_DOWN_CHECK) ||
+                (wrapperData->jState == WRAPPER_JSTATE_DOWN_FLUSH)) {
 
                 /* Signaled while we were already shutting down. */
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
@@ -411,11 +412,13 @@ const TCHAR* getSignalCodeDesc(int code) {
  * Describe a signal.  This is called from within signal handlers so NO MALLOCs are allowed here.
  */
 void descSignal(siginfo_t *sigInfo) {
+#ifdef SI_USER
     struct passwd *pw;
-#ifdef UNICODE
+ #ifdef UNICODE
     size_t req;
-#endif
+ #endif
     TCHAR *uName;
+#endif
 
     /* Not supported on all platforms */
     if (sigInfo == NULL) {
@@ -455,9 +458,9 @@ void descSignal(siginfo_t *sigInfo) {
             if (pw == NULL) {
                 uName = TEXT("<unknown>");
             } else {
-#ifndef UNICODE
+ #ifndef UNICODE
                 uName = pw->pw_name;
-#else
+ #else
                 req = mbstowcs(NULL, pw->pw_name, 0) + 1;
                 uName = malloc(req * sizeof(TCHAR));
                 if (!uName) {
@@ -465,20 +468,20 @@ void descSignal(siginfo_t *sigInfo) {
                     return;
                 }
                 mbstowcs(uName, pw->pw_name, req + 1);
-#endif
+ #endif
             }
 
             /* It appears that the getsid function was added in version 1.3.44 of the linux kernel. */
             log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG,
-#ifdef UNICODE
+ #ifdef UNICODE
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%S)"),
-#else
+ #else
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%s)"),
-#endif
+ #endif
                 sigInfo->si_pid, getsid(sigInfo->si_pid), sigInfo->si_uid, uName);
-#ifdef UNICODE
+ #ifdef UNICODE
             free(uName);
-#endif
+ #endif
         }
 #endif
     }
@@ -1031,21 +1034,10 @@ int wrapperSleep(int ms) {
  * Detaches the Java process so the Wrapper will if effect forget about it.
  */
 void wrapperDetachJava() {
-    int fd;
-    
     wrapperSetJavaState(WRAPPER_JSTATE_DOWN_CLEAN, 0, -1);
     
-    /* Redirect the pipes with the JVM to /dev/null so the JVM doesn't block when it writes to them. */
-    fd = _topen(TEXT("/dev/null"), O_RDWR, 0);
-    if (fd != -1) {
-        close(pipedes[STDOUT_FILENO]);
-        /*dup2(fd, pipedes[STDOUT_FILENO]); */
-        close(pipedes[STDERR_FILENO]);
-        /* dup2(fd, pipedes[STDERR_FILENO]); */
-        /*if ((fd != pipedes[STDOUT_FILENO]) && (fd != pipedes[STDERR_FILENO])) { */
-        close(fd);
-        /*}*/
-    }
+    /* Leave the stdout/stderr pipe in pipedes alone so we grab any remaining output.
+     *  They should have been redirected on the JVM side anyway. */
 }
 
 
@@ -1135,15 +1127,11 @@ int wrapperBuildJavaCommand() {
 void wrapperExecute() {
     pid_t proc;
 
-    /* Only allocate a pipe if we have not already done so. */
-    if (!pipeInitialized) {
-        /* Create the pipe. */
-        if (pipe(pipedes) < 0) {
-            log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
-                       TEXT("Could not init pipe: %s"), getLastErrorText());
-            return;
-        }
-        pipeInitialized = TRUE;
+    /* Create the pipe. */
+    if (pipe(pipedes) < 0) {
+        log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
+                   TEXT("Could not init pipe: %s"), getLastErrorText());
+        return;
     }
 
     /* Make sure the log file is closed before the Java process is created.  Failure to do
@@ -1187,18 +1175,24 @@ void wrapperExecute() {
              * TODO: Figure out a way to fix this.  Maybe using shared memory? */
 
             /* Send output to the pipe by dupicating the pipe fd and setting the copy as the stdout fd. */
-            if (dup2(pipedes[STDOUT_FILENO], STDOUT_FILENO) < 0) {
+            if (dup2(pipedes[PIPE_WRITE_END], STDOUT_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stdout: %s"), LOG_FORK_MARKER, getLastErrorText());
                 return;
             }
 
             /* Send errors to the pipe by dupicating the pipe fd and setting the copy as the stderr fd. */
-            if (dup2(pipedes[STDOUT_FILENO], STDERR_FILENO) < 0) {
+            if (dup2(pipedes[PIPE_WRITE_END], STDERR_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stderr: %s"), LOG_FORK_MARKER, getLastErrorText());
                 return;
             }
+            
+            /* Close both ends of the pipe as we have already duplicated the Write end for our purposes. */
+            close(pipedes[PIPE_READ_END]);
+            pipedes[PIPE_READ_END] = -1;
+            close(pipedes[PIPE_WRITE_END]);
+            pipedes[PIPE_WRITE_END] = -1;
 
             /* The pipedes array is global so do not close the pipes. */
             /* Child process: execute the JVM. */
@@ -1234,7 +1228,10 @@ void wrapperExecute() {
         } else {
             /* We are the parent side. */
             wrapperData->javaPID = proc;
-            jvmOut = pipedes[STDIN_FILENO];
+            
+            /* Close the write end as it is not used. */
+            close(pipedes[PIPE_WRITE_END]);
+            pipedes[PIPE_WRITE_END] = -1;
 
             /* Restore the auto close flag. */
             setLogfileAutoClose(wrapperData->logfileInactivityTimeout <= 0);
@@ -1243,12 +1240,12 @@ void wrapperExecute() {
 
             /* Mark our side of the pipe so that it won't block
              * and will close on exec, so new children won't see it. */
-            if (fcntl(jvmOut, F_SETFL, O_NONBLOCK) < 0) {
+            if (fcntl(pipedes[PIPE_READ_END], F_SETFL, O_NONBLOCK) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("Failed to set JVM output handle to non blocking mode: %s (%d)"),
                     getLastErrorText(), errno);
             }
-            if (fcntl(jvmOut, F_SETFD, FD_CLOEXEC) < 0) {
+            if (fcntl(pipedes[PIPE_READ_END], F_SETFD, FD_CLOEXEC) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("Failed to set JVM output handle to close on JVM exit: %s (%d)"),
                     getLastErrorText(), errno);
@@ -1401,9 +1398,10 @@ int wrapperGetProcessStatus(TICKS nowTicks, int sigChild) {
         res = WRAPPER_PROCESS_UP;
     } else if (retval < 0) {
         if (errno == ECHILD) {
-            if (wrapperData->jState == WRAPPER_JSTATE_DOWN_CHECK ||
-                wrapperData->jState == WRAPPER_JSTATE_DOWN_CLEAN ||
-                wrapperData->jState == WRAPPER_JSTATE_STOPPED) {
+            if ((wrapperData->jState == WRAPPER_JSTATE_DOWN_CHECK) ||
+                (wrapperData->jState == WRAPPER_JSTATE_DOWN_FLUSH) ||
+                (wrapperData->jState == WRAPPER_JSTATE_DOWN_CLEAN) ||
+                (wrapperData->jState == WRAPPER_JSTATE_STOPPED)) {
                 res = WRAPPER_PROCESS_DOWN;
                 wrapperJVMProcessExited(nowTicks, 0);
                 return res;
@@ -1470,7 +1468,7 @@ void wrapperReportStatus(int useLoggerQueue, int status, int errorCode, int wait
  * Returns TRUE if there were any problems, FALSE otherwise.
  */
 int wrapperReadChildOutputBlock(char *blockBuffer, int blockSize, int *readCount) {
-    if (jvmOut == -1) {
+    if (pipedes[PIPE_READ_END] == -1) {
         /* The child is not up. */
         *readCount = 0;
         return FALSE;
@@ -1481,15 +1479,15 @@ int wrapperReadChildOutputBlock(char *blockBuffer, int blockSize, int *readCount
      *  http://www.freebsd.org/cgi/query-pr.cgi?pr=kern/64313
      *
      * When linked with the pthreads library the O_NONBLOCK flag is being reset
-     *  on the jvmOut handle.  Not sure yet of the exact event that is causing
+     *  on the pipedes[PIPE_READ_END] handle.  Not sure yet of the exact event that is causing
      *  this, but once it happens reads will start to block even though calls
-     *  to fcntl(jvmOut, F_GETFL) say that the O_NONBLOCK flag is set.
-     * Calling fcntl(jvmOut, F_SETFL, O_NONBLOCK) again will set the flag back
+     *  to fcntl(pipedes[PIPE_READ_END], F_GETFL) say that the O_NONBLOCK flag is set.
+     * Calling fcntl(pipedes[PIPE_READ_END], F_SETFL, O_NONBLOCK) again will set the flag back
      *  again and cause it to start working correctly.  This may only need to
      *  be done once, however, because F_GETFL does not return the accurate
      *  state there is no reliable way to check.  Be safe and always set the
      *  flag. */
-    if (fcntl(jvmOut, F_SETFL, O_NONBLOCK) < 0) {
+    if (fcntl(pipedes[PIPE_READ_END], F_SETFL, O_NONBLOCK) < 0) {
         log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT(
             "Failed to set JVM output handle to non blocking mode to read child process output: %s (%d)"),
             getLastErrorText(), errno);
@@ -1498,9 +1496,8 @@ int wrapperReadChildOutputBlock(char *blockBuffer, int blockSize, int *readCount
 #endif
 
     /* Fill read buffer. */
-    *readCount = read(jvmOut, blockBuffer, blockSize);
-
-    if (*readCount <= 0) {
+    *readCount = read(pipedes[PIPE_READ_END], blockBuffer, blockSize);
+    if (*readCount < 0) {
         /* No more bytes available, return for now.  But make sure that this was not an error. */
         if (errno == EAGAIN) {
             /* Normal, the call would have blocked as there is no data available. */
@@ -1510,6 +1507,10 @@ int wrapperReadChildOutputBlock(char *blockBuffer, int blockSize, int *readCount
                 getLastErrorText(), errno);
             return TRUE;
         }
+    } else if (*readCount == 0) {
+        /* We reached the EOF.  This means that the other end of the pipe was closed. */
+        close(pipedes[PIPE_READ_END]);
+        pipedes[PIPE_READ_END] = -1;
     }
 
     return FALSE;
@@ -1517,9 +1518,6 @@ int wrapperReadChildOutputBlock(char *blockBuffer, int blockSize, int *readCount
 
 /**
  * Transform a program into a daemon.
- * Inspired by code from GNU monit, which in turn, was
- * inspired by code from Stephen A. Rago's book,
- * Unix System V Network Programming.
  *
  * The idea is to first fork, then make the child a session leader,
  * and then fork again, so that it, (the session group leader), can
@@ -1656,7 +1654,6 @@ int main(int argc, char **argv) {
     int i;
 #endif
     TCHAR *retLocale;
-    int localeSet;
 #ifdef UNICODE
     size_t req;
     TCHAR **argv;
@@ -1689,7 +1686,6 @@ int main(int argc, char **argv) {
 #if defined(UNICODE)
         free(retLocale);
 #endif
-        localeSet = TRUE;
     } else {
         /* TODO - We need to be careful about LANG here as it is not set on all systems. */
         /*
@@ -1699,7 +1695,6 @@ int main(int argc, char **argv) {
         free(envLang);
 #endif
         */
-        localeSet = FALSE;
     }
     if (wrapperInitialize()) {
         appExit(1, argc, argv);
@@ -1732,6 +1727,10 @@ int main(int argc, char **argv) {
         return 1; /* For compiler. */
     }
     wrapperLoadHostName();
+    if (!strcmpIgnoreCase(wrapperData->argCommand, TEXT("-translate"))) {
+        /* We want to disable all log output when a translation request is made. */
+        setSilentLogLevels();
+    }
     /* At this point, we have a command, confFile, and possibly additional arguments. */
     if (!strcmpIgnoreCase(wrapperData->argCommand, TEXT("?")) || !strcmpIgnoreCase(wrapperData->argCommand, TEXT("-help"))) {
         /* User asked for the usage. */
@@ -1769,8 +1768,13 @@ int main(int argc, char **argv) {
 
     /* Set the default umask of the Wrapper process. */
     umask(wrapperData->umask);
-
-    if (!strcmpIgnoreCase(wrapperData->argCommand, TEXT("c")) || !strcmpIgnoreCase(wrapperData->argCommand, TEXT("-console"))) {
+    if (!strcmpIgnoreCase(wrapperData->argCommand, TEXT("-translate"))) {
+        setSimpleLogLevels();
+        /* Print out the string so the caller sees it as its translated output. */
+        _tprintf(TEXT("%s"), argv[2]);
+        appExit(0, argc, argv);
+        return 0; /* For compiler. */
+    } else if (!strcmpIgnoreCase(wrapperData->argCommand, TEXT("c")) || !strcmpIgnoreCase(wrapperData->argCommand, TEXT("-console"))) {
         /* Run as a console application */
 
         /* fork to a Daemonized process if configured to do so. */
