@@ -189,6 +189,15 @@ public class WrapperJarApp
         
         Attributes attributes = manifest.getMainAttributes();
         String mainClassName = attributes.getValue( "Main-Class" );
+        if ( mainClassName == null ) 
+        {
+            m_outError.println( WrapperManager.getRes().getString(
+                    "The Main-Class was not specified correctly in the jar file. Please make sure all required meta information is being set." ) );
+            /* no main class, no chance this will ever do something sensible, so stop here now */
+            WrapperManager.stop( 1 );
+            return;  // Will not get here
+
+        }
         String classPath = attributes.getValue( "Class-Path" );
 
         if ( WrapperManager.isDebugEnabled() )
@@ -379,7 +388,17 @@ public class WrapperJarApp
             {
                 m_outDebug.println( WrapperManager.getRes().getString("invoking main method" ) );
             }
-            m_mainMethod.invoke( null, new Object[] { m_appArgs } );
+            
+            try
+            {
+                m_mainMethod.invoke( null, new Object[] { m_appArgs } );
+            }
+            finally
+            {
+                // Make sure the rest of this thread does not fall behind the application.
+                Thread.currentThread().setPriority( Thread.MAX_PRIORITY );
+            }
+            
             if ( WrapperManager.isDebugEnabled() )
             {
                 m_outDebug.println(  WrapperManager.getRes().getString("main method completed" ) );
@@ -504,6 +523,9 @@ public class WrapperJarApp
             m_appArgs = args;
             mainThread.start();
             
+            // Make sure the rest of this thread does not fall behind the application.
+            Thread.currentThread().setPriority( Thread.MAX_PRIORITY );
+            
             // To avoid problems with the main thread starting slowly on heavily loaded systems,
             //  do not continue until the thread has actually started.
             while ( !m_mainStarted )
